<?php
function metatags_quick_admin_settings() {
  $current_settings = variable_get('metatags_quick_settings', _metatags_quick_settings_default());
  $module_path = drupal_get_path('module', 'metatags_quick');
  $fields = field_info_fields();
  $metatags_found = FALSE;  
  
  $form['standard_tags'] = array(
    '#type' => 'fieldset',
    '#title' => t('Create and attach'),
    '#description' => t('Create and attach meta tags to existing entities'),
    '#collapsible' => TRUE,
    '#attached' => array(
      'js' => array($module_path . '/js/admin.js'),
      'css' => array($module_path . '/css/admin.css'),
    ),
  );
  
  $field_instances = field_info_instances();
  
  // Build the sortable table header.
  $header = array(
    'title' => array('data' => t('Entity')),
    'keywords' => array('data' => t('Keywords')),
    'description' => array('data' => t('Description')),
    'copyright' => array('data' => t('Copyright')),
    'all' => array('data' => t('Select all'))
  );
  
  foreach (field_info_bundles() as $entity_type => $bundles) {
    $entity_info = entity_get_info($entity_type);
    if (!$entity_info['fieldable']) {
      continue;
    }
    $options[$entity_type] = array(
      'title' => array(
        'class' => array('entity_type_collapsible', 'entity_type_collapsed', "entity_name_$entity_type"),
        'data' => $entity_info['label']
      ),
      'keywords' => array(
        'data' => array(
          '#type' => 'checkbox',
          '#return_value' => 1,
          '#checked' => FALSE,
        ),
      ),
      'description' => array(
        'data' => array(
          '#type' => 'checkbox',
          '#return_value' => 1,
          '#checked' => FALSE,
        ),
      ),
      'copyright' => array(
        'data' => array(
          '#type' => 'checkbox',
          '#return_value' => 1,
          '#checked' => FALSE,
        ),
      )
    );
    foreach ($bundles as $key => $bundle) {
      // Which meta tags are set for this bundle?
      $meta_set = array();
      foreach ($field_instances[$entity_type][$key] as $bundle_instance) {
        if ($bundle_instance['widget']['module'] == 'metatags_quick') {
          $field_info = field_info_field_by_id($bundle_instance['field_id']);
          $meta_set[$field_info['settings']['meta_name']] = 1;
        }        
      }
      
      $options[$entity_type . '_' . $key] = array(
        'class' => array('entity_type_children', "entity_child_$entity_type"),
        'style' => 'display: none',
        'data' => array(
          'title' => array(
            'class' => array('entity_type_child_title'),
            'data' => $bundle['label'],
          ),
          'keywords' => array(
            'data' => array(
              '#type' => 'checkbox',
              '#return_value' => 1,
              '#checked' => !empty($meta_set['keywords']),
            ),
          ),
          'description' => array(
            'data' => array(
              '#type' => 'checkbox',
              '#return_value' => 1,
              '#checked' => !empty($meta_set['description']),
            ),
          ),
          'copyright' => array(
            'data' => array(
              '#type' => 'checkbox',
              '#return_value' => 1,
              '#checked' => !empty($meta_set['checked']),
            ),
          ),
        ),
      );
    }
  }
  
  $form['standard_tags']['existing'] = array(
    '#theme' => 'table',
    '#header' => $header,
    '#rows' => $options,
    '#empty' => t('No content available.'),
  );
  
  $form['standard_tags']['basic_init_op'] = array(
    '#type' => 'submit',
    '#value' => t('Create basic meta tags'),
  );
  
  $form['front_settings'] = array(
    '#type' => 'fieldset',
    '#title' => t('Front page meta tags'),
    '#collapsible' => TRUE,
    '#collapsed' => TRUE,
  );

  foreach ($fields as $key => $field) {
    if ($field['module'] != 'metatags_quick') {
      continue;
    }
    $metatags_found = TRUE;
    $form_key = $field['settings']['meta_name'];
    $field_form = array(
      '#title' => $key,
      '#type' => 'textarea',
      '#default_value' => empty($current_settings['front_values'][$form_key]) ? '' : $current_settings['front_values'][$form_key],
    );
    $form['front_settings'][$form_key] = $field_form;
  }
  if (!$metatags_found) {
    $form['front_settings']['basic_init'] = array(
      '#markup' => t('No meta tags found in your installation') . '<br/>',
    );
  }
  
  $form['front_settings']['use_front'] = array(
    '#type' => 'checkbox',
    '#title' => t('Use front page meta tags'),
    '#description' => t('Don\'t use that option if your front page is a fieldable entity (i.e. single node, taxonomy term etc.)'),
    '#default_value' => $current_settings['use_front'],
    '#return_value' => 1,
  );
  
  
  $form['op'] = array(
    '#value' => t('Submit'),
    '#type' => 'submit', 
  );
  return $form;
}

function metatags_quick_admin_settings_submit($form, &$form_state) {
  $new_settings = array(
    'use_front' => !empty($form_state['values']['use_front']),
  );
  if (isset($form_state['values']['basic_init_op'])) {
    _metatags_quick_init_basic_fields();
  }
  else {
    foreach (field_info_fields() as $field_name => $field) {
      if ($field['module'] != 'metatags_quick') {
        continue;
      }
      $form_key = $field['settings']['meta_name'];
      $new_settings['front_values'][$form_key] = $form_state['values'][$form_key];
    }
  }
  variable_set('metatags_quick_settings', $new_settings);
  drupal_set_message(t('Meta tags (quick) settings saved'), 'status');
}

/*
 * Path-based meta tags editing form.
 * @param string $path - path being edited.
 */
function metatags_quick_admin_path() {
  global $language;
  $args = func_get_args();
  $form = array();
  if (count($args) > 2) {
    $path = $args[2];
  }
  else {
    return $form;
  }
  
  $form['meta_tags'] = array(
    '#type' => 'fieldset',
    '#title' => t('Edit meta tags'),
    '#collapsible' => TRUE,
  );
  $fields = field_info_fields();
  $metatags_found = FALSE;
  $meta_fields = array();
  foreach ($fields as $key => $field) {
    if ($field['module'] != 'metatags_quick') {
      continue;
    }
    $metatags_found = TRUE;
    $form_key = $field['settings']['meta_name'];
    $meta_fields[$field['id']] = array('key' => $key) ;
  }
  
  if (!$metatags_found) {
    $form['meta_tags']['basic_init'] = array(
      '#markup' => t('No meta tags found in your installation'),
    );
    $form['meta_tags']['basic_init_op'] = array(
      '#type' => 'submit',
      '#value' => t('Create basic meta tags'),
    );
  }
  else {  
    $current_values = db_select('metatags_quick_path_based', 'pv')
      ->condition('lang', $language->language)
      ->condition('path', $path)
      ->fields('pv', array('fid', 'value'))
      ->execute();
    
    foreach ($current_values as $row) {
      $meta_fields[$row->fid]['value'] = $row->value;
    }
    
    foreach ($meta_fields as $key => $settings) {
      $form_key = $fields[$meta_fields[$key]['key']]['settings']['meta_name'];
      $form_key = $key;
      $field_form = array(
        '#title' => $meta_fields[$key]['key'],
        '#type' => 'textarea',
        '#default_value' => isset($meta_fields[$key]['value']) ? $meta_fields[$key]['value'] : '',
      );
      $form['meta_tags'][$form_key] = $field_form;
      $form['meta_tags']["{$form_key}_set"] = array(
        '#type' => 'hidden',
        '#value' => isset($meta_fields[$key]['value']) ? '1' : '0',
      );
    }
  }
  $form['op'] = array(
    '#value' => t('Submit'),
    '#type' => 'submit', 
  );  
  return $form;
}

function metatags_quick_admin_path_submit($form, &$form_state) {
  $path = $form_state['build_info']['args'][0];
  if (!$path) {
    form_set_error();
    drupal_set_message(t('Wrong path'), 'error');
    return;
  }
  
  // Now save data.
  $fields = field_info_fields();
  foreach ($fields as $key => $field) {
    if ($field['module'] != 'metatags_quick') {
      continue;
    }
    $form_key = $field['id'];
    if ($form_state['values']["{$form_key}_set"]) {
      db_update('metatags_quick_path_based')
        ->condition('lang', $GLOBALS['language']->language)
        ->condition('path', $path)
        ->condition('fid', $field['id'])
        ->fields(array('value' => $form_state['values'][$form_key]))
        ->execute();
    }
    else {
      db_insert('metatags_quick_path_based')->fields(array(
        'lang' => $GLOBALS['language']->language,
        'path' => $path,
        'fid' => $field['id'],
        'value' => $form_state['values'][$form_key],
      ))->execute();
    }
  }
  
  drupal_set_message('Meta tags updated', 'status');
}
